sample.ui.ChatPage = Backbone.View.extend({

    connection: null,

    events: {
        "keydown #chat_message": 'onKeyDown'
    },

    initialize: function() {
        $('.container').append('<div class="page chat"></div>');

        this.setElement('.chat');

        this.render();

        var self = this;


    },

    render: function() {
        var self = this;

        $.ajax({
            url: 'app/templates/chat.tpl',
            dataType: 'text'
        }).done(function(data) {
                self.$el.html(data);

                self.initChat();

                sample.core.state.bind('change:content', function() {
                    if (self.connection) {
                        self.connection.send(JSON.stringify({'type': 'history'}));
                    }

                });

                sample.core.state.bind('change:userName', function() {

                    if (!sample.core.state.get('userName')) {
                        $('.chat_status_line').hide();
                    } else {
                        $('.chat_status_line').show();
                    }

                });
            })
    },

    initChat: function() {
        var container = $('#chat_history');

        // if user is running mozilla then use it's built-in WebSocket
        window.WebSocket = window.WebSocket || window.MozWebSocket;

        // open connection
        this.connection = new WebSocket('ws://192.168.100.48:1337');

        this.connection.onopen = function () {
            // onOpen here
        };

        var self = this;

        // most important part - incoming messages
        this.connection.onmessage = function (message) {
            // try to parse JSON message. Because we know that the server always returns
            // JSON this should work without any problem but we should make sure that
            // the massage is not chunked or otherwise damaged.
            console.log(message);

            try {
                var json = JSON.parse(message.data);
            } catch (e) {
                console.log('This doesn\'t look like a valid JSON: ', message.data);
                return;
            }

            // NOTE: if you're not sure about the JSON structure
            // check the server source code above
            if (json.type === 'history') { // entire message history
                $('#chat_history').html('');

                // insert every single message to the chat window
                for (var i=0; i < json.data.length; i++) {
                    self.addMessage(json.data[i].room, json.data[i].author, json.data[i].text,
                        json.data[i].color, new Date(json.data[i].time));
                }
            } else if (json.type === 'message') { // it's a single message
                $('#chat_message').removeAttr('disabled'); // let the user write another message
                self.addMessage(json.data.room, json.data.author, json.data.text,
                    json.data.color, new Date(json.data.time));
            } else {
                console.log('Hmm..., I\'ve never seen JSON like this: ', json);
            }
        };
    },

    onKeyDown: function(e) {
        if (e.keyCode === 13) {
            var msg = {
                'room': sample.core.state.get('content'),
                'user': sample.core.state.get('userName'),
                'text':  $(e.target).val()
            };

            if (!$(e.target).val()) {
                return;
            }
            // send the message as an ordinary text
            this.connection.send(JSON.stringify(msg));
            // disable the input field to make the user wait until server
            // sends back response
            $('#chat_message').attr('disabled', 'disabled');
            $('#chat_message').val('');
        }
    },

    addMessage:function(room, author, message, color, dt) {

        if (room == sample.core.state.get('content')) {
            $('#chat_history').append('<p><span style="color:' + color + '">' + author + '</span> @ ' +
                + (dt.getHours() < 10 ? '0' + dt.getHours() : dt.getHours()) + ':'
                + (dt.getMinutes() < 10 ? '0' + dt.getMinutes() : dt.getMinutes())
                + ': ' + message + '</p>');
        }
    }
});